% This matlab code computes the numbers in table A4 
% in the online appendix to "Quasi-Bayesian model selection" 
% by Atsushi Inoue and Mototsugu Shintani

% Date       Programmer     Description
% 10/21/2013  Atsushi Inoue  Original code
% 12/15/2016  Atsushi Inoue  Revised
% 10/16/2017 Atsushi Inoue  Revised

% Initialization
B = 1000;  % Number of bootstrap replications used to estimate covariance matrix of impulse responses
options=optimoptions(@fminunc,'Algorithm','quasi-newton','HessUpdate','bfgs','FunvalCheck','on','Display','none');
warning('off')
M = 100;   % Number of Monte Carlo replications
N = 50000; % Number of MCMC draws; only second half is used to compute quasi-marginal likelihood

fid=fopen('tableA4.tex','a');
fprintf(fid,'\\begin{tabular}{l|ccccccccccc}\\hline\n');
fprintf(fid,'& & \\multicolumn{4}{c}{Geweke} & \\multicolumn{4}{c}{SWZ} & \\multicolumn{2}{c}{CJ} \\\\ \\cline{2-11} \n');
fprintf(fid,' & & \\tau=0.5$ & $\\tau=0.9$ & $q=0.5$ & $q=0.9$ & &  \\\\ \\cline{2-11} \n');
fprintf(fid,'T & true & mean & SD & mean & SD & mean & SD & mean & SD & mean & SD \\\\ \\hline \n');
fclose(fid);

% Monte Carlo experiment parameters
thetaA       = parameters;
thetaA.value = [0.5 0.5 1 1]; 
thetaA.name  = {'alpha ','kappa ','sigma ','zeta  '}; 

for experiment = 1:6
    switch experiment
        case 1
            thetaA.estimation = [0 1 1 0];
            c0A = 1; c1A = 2; 	
	    indx = [1 1 1 1 1];
        case 2
            thetaA.estimation = [0 0 1 0];
            c0A = 2; c1A = 4; 	
	    indx = [1 1 1 1 1];
        case 3
	    thetaA.estimation = [0 1 1 0];
            c0A = 1; c1A = 2; 	
	    indx = [0 1 0 1 1];
        case 4
            thetaA.estimation = [0 0 1 0];
            c0A = 2; c1A = 4; 	
	    indx = [0 1 0 1 1];
        case 5
            thetaA.estimation = [1 0 0 1];
            sigma = 1; alpha = 0.5; zeta = 1;
            kappa = (1-alpha)*(1-0.99*alpha)*zeta/alpha;
	    thetaA.value = [alpha kappa sigma zeta];
            c0A = 2; c1A = 4; 
            indx = [1 1 1 1 1];
        case 6
            thetaA.estimation = [1 0 0 1];
            sigma = 1; alpha = 0.5; zeta = 1;
            kappa = (1-alpha)*(1-0.99*alpha)*zeta/alpha;
	    thetaA.value = [alpha kappa sigma zeta];
            c0A = 2; c1A = 4; 
            indx = [1 1 1 1 1];
    end
    nA = sum(thetaA.estimation);
    kappa = thetaA.value(2);
    sigma = thetaA.value(3);
    Omega0 = [1 0 -sigma;kappa 1 -sigma*kappa;0 0 1];
    Omega0 = Omega0*Omega0';
    P0     = chol(Omega0)';
    if experiment==6
        thetaA.value(3)=0.5;
    end
    thetaA0 = thetaA.value(thetaA.estimation==1);
    fid = fopen('tableA4.tex','a');
    fprintf(fid,'\\multicolumn{12}{c}{Case %2.0f} \\\\ \\hline\n',experiment); 
    fclose(fid);
    for T=[50 100 200]
        rng('default');
        %disp('1. Generate data and bootstrap the covariance matrix.')
        y        = randn(T,3)*P0';
        Omegahat = y'*y/T;
        gammahat = [Omegahat(1,1);Omegahat(2,1);Omegahat(3,1);Omegahat(2,2);Omegahat(2,3)];
        gammahat = gammahat(indx==1);
        W = [];
        for i=1:B
            ystar        = y(floor(rand(T,1)*T)+1,:);	
            Omegahatstar = cov(ystar,1); % recentering
            gammahatstar = [Omegahatstar(1,1);Omegahatstar(2,1);Omegahatstar(3,1);Omegahatstar(2,2);Omegahatstar(2,3)];
            W            = [W; gammahatstar(indx==1)'];
        end
        W  = inv(cov(W)); % optimal weighting matrix
        %W  = inv(diag(diag(cov(W)))); % diagonal weighting matrix

        save temp gammahat W indx;
        switch experiment
            case 1
                qml = integral2(@cmd1,-20,20,-20,20);
            case 2
                qml = integral(@cmd2,-20,20);
            case 3
                 qml = integral2(@cmd3,-20,20,-20,20);
            case 4
                qml = integral(@cmd4,-50,50);
            case 5
                qml = integral2(@cmd5,0,1,-20,20);
            case 6
                qml = integral2(@cmd6,0,1,-20,20);
        end
        qml = log(qml)
        [modeAopt,f,SigmaAopt] = findmode(gammahat,indx,options,thetaA,W,thetaA0);
        ml = zeros(M,5);
        i=1;
        while i<=M
            MCMCAopt = mcmc(c0A,c1A,gammahat,indx,N,SigmaAopt,thetaA,W,modeAopt);
            ml(i,:) = mdd(gammahat,indx,MCMCAopt,thetaA,W);
            if (sum(isinf(ml(i,:)))>0)|(sum(isnan(ml(i,:)))>0)
                continue
            end
            i=i+1;
        end
        ml = [mean(ml);std(ml)];
        ml = reshape(ml,1,10);
        fid=fopen('tableA4.tex','a');
        fprintf(fid,' %3.0f & %5.3f & %5.3f & [%5.3f] & %5.3f & [%5.3f] & %5.3f & [%5.3f] & %5.3f & [%5.3f] & %5.3f & [%5.3f] \\\\ \n',T,qml,ml);
        fclose(fid);
    end
    fid = fopen('tableA4.tex','a');
    fprintf(fid,'\\hline\n'); 
    fclose(fid);
end
fid=fopen('tableA4.tex','a');
fprintf(fid,'\\end{tabular}\n');
fclose(fid);
